@@ -72,7 +72,7 @@ NULL
 #' \code{MultiAssayExperiment} object
 #' @slot drops A metadata \code{list} of dropped information
 #'
-#' @param x A \code{MultiAssayExperiment} object
+#' @param object,x A \code{MultiAssayExperiment} object
 #' @param ... Additional arguments for supporting functions. See details.
 #'
 #' @return A \code{MultiAssayExperiment} object
@@ -350,7 +350,6 @@ S4Vectors::setValidity2("MultiAssayExperiment", .validMultiAssayExperiment)
 #' @exportMethod show
 #' @describeIn MultiAssayExperiment Show method for a
 #' \code{MultiAssayExperiment}
-#' @param object A \code{MultiAssayExperiment} object
 setMethod("show", "MultiAssayExperiment", function(object) {
     if (.hasOldAPI(object)) {
         stop("MultiAssayExperiment is outdated, please run updateObject()")
@@ -361,7 +360,6 @@ setMethod("show", "MultiAssayExperiment", function(object) {
     if (length(o_names) == 0L) {
         o_names <- "none"
     }
-    classes <- vapply(experiments(object), class, character(1))
     c_elist <- class(experiments(object))
     c_mp <- class(colData(object))
     c_sm <- class(sampleMap(object))
@@ -374,7 +372,7 @@ setMethod("show", "MultiAssayExperiment", function(object) {
                       "user-defined names")),
         ifelse(length(o_len) == 0L, "or", "and"),
         ifelse(length(o_len) == 0L, "classes.",
-               ifelse(length(classes) == 1L,
+               ifelse(o_len == 1L,
                       "respective class.\n", "respective classes.\n")),
         "Containing an ")
     show(experiments(object))
@@ -423,8 +421,7 @@ setMethod("show", "MultiAssayExperiment", function(object) {
 #'     \item `$<-`: A vector to replace the indicated column in \code{colData}
 #' }
 #'
-#' @param x A \code{MultiAssayExperiment} object
-#' @param object A \code{MultiAssayExperiment} object
+#' @param object,x A \code{MultiAssayExperiment} object
 #' @param name A column in \code{colData}
 #' @param value See details.
 #' @param ... Argument not in use
@@ -442,6 +439,7 @@ NULL
 ### Accessor methods
 ###
 
+#' @export
 setGeneric("sampleMap", function(x) standardGeneric("sampleMap"))
 
 #' @exportMethod sampleMap
@@ -458,7 +456,6 @@ setMethod("experiments", "MultiAssayExperiment", function(x)
     getElement(x, "ExperimentList"))
 
 
-#' @export
 #' @exportMethod colData
 #' @rdname MultiAssayExperiment-methods
 setMethod("colData", "MultiAssayExperiment", function(x, ...) {
@@ -490,6 +487,7 @@ setMethod("names", "MultiAssayExperiment", function(x)
 ### Replacers
 ###
 
+#' @export
 setGeneric("sampleMap<-", function(object, value) {
     standardGeneric("sampleMap<-")
 })
@@ -497,11 +495,33 @@ setGeneric("sampleMap<-", function(object, value) {
 #' @exportMethod sampleMap<-
 #' @rdname MultiAssayExperiment-methods
 setReplaceMethod("sampleMap", c("MultiAssayExperiment", "DataFrame"),
-                function(object, value) {
-                    slot(object, "sampleMap") <- value
-                    return(object)
-                })
+    function(object, value) {
+        if (isEmpty(value))
+            value <- DataFrame(assay = factor(), primary = character(),
+                colname = character())
+        rebliss <- .harmonize(experiments(object),
+            colData(object),
+            value)
 
+        BiocGenerics:::replaceSlots(object,
+            ExperimentList = rebliss[["experiments"]],
+            colData = rebliss[["colData"]],
+            sampleMap = rebliss[["sampleMap"]],
+            check = FALSE
+        )
+    }
+)
+
+#' @rdname MultiAssayExperiment-methods
+setReplaceMethod("sampleMap", c("MultiAssayExperiment", "ANY"),
+    function(object, value) {
+        stopifnot(is.data.frame(value))
+        value <- as(value, "DataFrame")
+        `sampleMap<-`(object, value)
+    }
+)
+
+#' @export
 setGeneric("experiments<-", function(object, value)
     standardGeneric("experiments<-"))
 
@@ -509,28 +529,45 @@ setGeneric("experiments<-", function(object, value)
 #' @rdname MultiAssayExperiment-methods
 setReplaceMethod("experiments", c("MultiAssayExperiment", "ExperimentList"),
     function(object, value) {
-        if (!length(value)) {
-            slot(object, "ExperimentList") <- value
-            return(object)
-        }
         rebliss <- .harmonize(value,
             colData(object),
             sampleMap(object))
+
         BiocGenerics:::replaceSlots(object,
             ExperimentList = rebliss[["experiments"]],
             colData = rebliss[["colData"]],
             sampleMap = rebliss[["sampleMap"]],
             check = FALSE
         )
-    })
+    }
+)
 
 #' @exportMethod colData<-
 #' @rdname MultiAssayExperiment-methods
 setReplaceMethod("colData", c("MultiAssayExperiment", "DataFrame"),
     function(x, value) {
-        slot(x, "colData") <- value
-        return(x)
-    })
+        rebliss <- .harmonize(experiments(x),
+            value,
+            sampleMap(x))
+
+        BiocGenerics:::replaceSlots(x,
+            ExperimentList = rebliss[["experiments"]],
+            colData = rebliss[["colData"]],
+            sampleMap = rebliss[["sampleMap"]],
+            check = FALSE
+        )
+    }
+)
+
+#' @rdname MultiAssayExperiment-methods
+setReplaceMethod("colData", c("MultiAssayExperiment", "ANY"),
+    function(x, value) {
+        stopifnot(is.data.frame(value))
+        value <- as(value, "DataFrame")
+        `colData<-`(x, value)
+    }
+)
+
 
 .rearrangeMap <- function(sampMap) {
     return(DataFrame(assay = factor(sampMap[["assayname"]]),
@@ -685,14 +722,10 @@ S4Vectors::setValidity2("MatchedAssayExperiment", .validMatchedAssayExperiment)
 #' @export MatchedAssayExperiment
 MatchedAssayExperiment <- function(...) {
     listData <- list(...)
-    if (length(listData) == 1L) {
-        if (is(listData[[1L]], "MultiAssayExperiment"))
+    if (length(listData) && is(listData[[1L]], "MultiAssayExperiment"))
             multiassay <- listData[[1L]]
-        else
-            stop("Provide a 'MultiAssayExperiment' or its basic components")
-    } else {
+    else
         multiassay <- MultiAssayExperiment(...)
-    }
     multiassay <- .doMatching(multiassay)
     new("MatchedAssayExperiment", multiassay)
 }
@@ -702,14 +735,6 @@ setAs("MultiAssayExperiment", "MatchedAssayExperiment", function(from) {
     new("MatchedAssayExperiment", from)
 })
 
-setGeneric("exportClass",
-    function(object, dir = tempdir(), fmt, ext, match = FALSE,
-        verbose = TRUE, ...) {
-        standardGeneric("exportClass")
-    }
-)
-
-
 .metasize <- function(metlist) {
     atmos <- vapply(metlist, is.atomic, logical(1L))
     sum(any(atmos), !atmos)
@@ -723,18 +748,20 @@ setGeneric("exportClass",
         col.names = cols, qmethod = qme)
 }
 
-.sortMetadata <- function(object, dir, fmt, ext, ...) {
-    objname <- as.character(substitute(object))
+.sortMetadata <- function(object, objname, dir, fmt, ext, ...) {
     metas <- metadata(object)
     stopifnot(is.list(metas))
     atmos <- vapply(metas, is.atomic, logical(1L))
     metatxt <- metas[atmos]
 
-    fpath <- file.path(dir, paste0(objname, "_META_0", ext))
-    if (!is.function(fmt))
-        fmt <- .chr2fxn(fmt)
-    fmt(as.data.frame(metatxt), fpath, ...)
-    fnames <- fpath
+    fnames <- character(0L)
+    if (length(metatxt)) {
+        fpath <- file.path(dir, paste0(objname, "_META_0", ext))
+        if (!is.function(fmt))
+            fmt <- .chr2fxn(fmt)
+        fmt(as.data.frame(metatxt), fpath, ...)
+        fnames <- fpath
+    }
     if (any(!atmos)) {
         nonato <- metas[!atmos]
         nonatos <- seq_along(nonato)
@@ -751,8 +778,16 @@ setGeneric("exportClass",
     fnames
 }
 
-#' @describeIn MultiAssayExperiment Export data from class to a series of text
-#'     files
+#' @export
+setGeneric("exportClass",
+    function(object, dir = tempdir(), fmt, ext, match = FALSE,
+        verbose = TRUE, ...) {
+        standardGeneric("exportClass")
+    }
+)
+
+#' @describeIn MultiAssayExperiment Export data from class to a series
+#'     of text files
 #'
 #' @param dir character(1) A directory for saving exported data (default:
 #'     `tempdir()`)
@@ -766,6 +801,10 @@ setGeneric("exportClass",
 #' @param match logical(1) Whether to coerce the current object to a
 #'     'MatchedAssayExperiment' object (default: FALSE)
 #'
+#' @param verbose logical(1) Whether to print additional information (default
+#'     TRUE)
+#'
+#' @aliases exportClass
 #' @exportMethod exportClass
 setMethod("exportClass", "MultiAssayExperiment",
     function(object, dir = tempdir(), fmt, ext, match = FALSE,
@@ -804,7 +843,7 @@ setMethod("exportClass", "MultiAssayExperiment",
         lists <- c(alists, list(coldat = as.data.frame(colData(object)),
             sampmap = as.data.frame(sampleMap(object))))
 
-        metafs <- .sortMetadata(object, dir, fmt, ext)
+        metafs <- .sortMetadata(object, objname, dir, fmt, ext)
 
         invisible(Map(function(fname, lobject) {
             fmt(lobject, fname, ...)
