@@ -283,12 +283,13 @@ setMethod("mergeReplicates", "ANY",
 .longFormatElist <- function(object, i) {
     if (!is(object, "ExperimentList"))
         stop("<internal> Not an 'ExperimentList' input")
-    objnames <- stats::setNames(names(object), names(object))
-    lapply(objnames, function(nameidx, flatBox) {
-        data.frame(assay = nameidx,
-            .longFormatANY(flatBox[[nameidx]], i = i),
+    samelength <- identical(length(object), length(i))
+    if (!samelength && identical(length(i), 1L))
+        i <- rep(i, length(object))
+    mapply(function(obj, obname, idx) {
+        data.frame(assay = obname, .longFormatANY(obj, i = idx),
             stringsAsFactors = FALSE)
-        }, flatBox = object)
+        }, obj = object, obname = names(object), idx = i, SIMPLIFY = FALSE)
 }
 
 .matchAddColData <- function(reshaped, colData, colDataCols) {
@@ -330,10 +331,17 @@ setMethod("mergeReplicates", "ANY",
 #' argument in the assay method.
 #'
 #' @param object Any supported class object
+#'
 #' @param colDataCols A \code{character}, \code{logical}, or \code{numeric}
-#' index for \code{colData} columns to be included
+#'     index for \code{colData} columns to be included
+#'
 #' @param i The assay indicator for \linkS4class{SummarizedExperiment}
-#' objects (default 1L)
+#'     objects. A vector input is supported in the case that the
+#'     SummarizedExperiment object(s) has more than one assay (default 1L)
+#'
+#' @param mode String indicating how \linkS4class{MultiAssayExperiment}
+#'     column-level metadata should be added to the
+#'     \linkS4class{SummarizedExperiment} \code{colData}.
 #'
 #' @export longFormat
 longFormat <- function(object, colDataCols = NULL, i = 1L) {
@@ -537,24 +545,60 @@ setMethod("hasRowRanges", "ExperimentList", function(x) {
 #' \linkS4class{SummarizedExperiment} and \code{RaggedExperiment}
 #' classes.
 #'
+#' The setting of \code{mode} determines how the \code{\link{colData}}
+#' is added. If \code{mode="append"}, the \linkS4class{MultiAssayExperiment}
+#' metadata is appended onto that of the \linkS4class{SummarizedExperiment}.
+#' If any fields are duplicated by name, the values in the \linkS4class{SummarizedExperiment}
+#' are retained, with a warning emitted if the values are different.
+#' For \code{mode="replace"}, the \linkS4class{MultiAssayExperiment}
+#' metadata replaces that of the \linkS4class{SummarizedExperiment},
+#' while for \code{mode="none"}, no replacement or appending is performed.
+#'
 #' @export getWithColData
-getWithColData <- function(x, i) {
+getWithColData <- function(x, i, mode=c("append", "replace")) {
     if (!is(x, "MultiAssayExperiment"))
         stop("Provide a MultiAssayExperiment as input")
 
     stopifnot(is.numeric(i) || is.character(i),
-        identical(length(i), 1L), !is.na(i), !is.logical(i))
+        identical(length(i), 1L), !is.na(i), !is.logical(i),
+        is.character(mode), !is.na(mode), !is.logical(mode))
 
     mae <- x[, , i, drop = TRUE]
-    cdat <- colData(mae)
+    prims <- sampleMap(mae)[["primary"]]
+    if (anyDuplicated(prims))
+        warning("Duplicating colData rows due to replicates in 'replicated(x)'",
+            call. = FALSE)
+    expanded <- colData(mae)[prims, , drop = FALSE]
     exObj <- mae[[1L]]
+
     tryCatch({
-        colData(exObj) <- cdat
+        existing <- colData(exObj)
     }, error = function(e) {
         stop(
-            "Extracted class does not support 'colData<-':",
+            "Extracted class does not support 'colData':",
             "\n    ", conditionMessage(e), call. = FALSE
         )
     })
+
+    mode <- match.arg(mode)
+    if (identical(mode, "replace") || !length(existing))
+            colData(exObj) <- expanded
+    else if (identical(mode, "append")) {
+        # Prune out duplicate entries
+        common <- intersect(colnames(existing), colnames(expanded))
+        if (length(common)) {
+            warnid <- vapply(common, function(varname, x, y) {
+                !identical(x[[varname]], y[[varname]])
+            }, logical(1L), x = existing, y = expanded)
+            .warning(
+                "Ignoring redundant column names in 'colData(x)': ",
+                paste(common[warnid], collapse = ", ")
+            )
+        }
+        leftovers <- expanded[, setdiff(colnames(expanded), common), drop=FALSE]
+        colData(exObj) <- cbind(existing, leftovers)
+    }
+
     exObj
 }
+
